#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform struct DirLight {
	vec3 color;
	vec3 direction;
};

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
};

uniform unsigned int dir_lights_NR;
uniform unsigned int point_lights_NR;
uniform DirLight dirLights[20];
uniform PointLight pointLights[20];

uniform vec3 baseColorDiffuse;
uniform vec3 baseColorSpecular;
uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular


uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;

const float levels = 8;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}


vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

vec3 celShade(vec3 c){
	vec3 hsv_c = rgb2hsv(c);
	vec3 rgb_c = hsv2rgb(c);
	return rgb_c;
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb + baseColorDiffuse;
	vec3 specColor = texture(specularTexture, vert.uv).rgb;
	color = vec4((texColor * materialCoefficients.x), 1);
	
	// add directional light contribution
	for(int i=0; i < dir_lights_NR; i++){
	color.rgb += phong(n, -dirLights[i].direction, v, dirLights[i].color * texColor, materialCoefficients.y, dirLights[i].color * specColor, materialCoefficients.z, specularAlpha, false, vec3(0));
	}		
	// add point light contribution
	for(int i=0; i < point_lights_NR; i++){
	color.rgb += phong(n, pointLights[i].position - vert.position_world, v, pointLights[i].color * texColor, materialCoefficients.y, pointLights[i].color * specColor, materialCoefficients.z, specularAlpha, true, pointLights[i].attenuation);
	}

	//color.rgb = rgb2hsv(color.rgb);
	//float level = floor(color.rgb.z * levels);
	//color.rgb.z = level/levels;
	//color.rgb = hsv2rgb(color.rgb);
}